// Copyright (c) 2002 Cunningham & Cunningham, Inc.
// Released under the terms of the GNU General Public License version 2 or later.

using System;
using System.Globalization;
using System.IO;

namespace eg.music
{

	public class Music 
	{

		internal static Music[] library = {};
		internal static Music looking = null;
		internal static Music playing = null;

		public string title;
		public string artist;
		public string album;
		public string genre;
		public long size;
		public int seconds;
		public int trackNumber;
		public int trackCount;
		public int year;
		public DateTime date;
		public bool selected = false;

		// Accessors ////////////////////////////////

		public string track() 
		{
			return trackNumber + " of " + trackCount;
		}

		public double time() 
		{
			return Math.Round(seconds / 0.6) / 100.0;
		}

		public override string ToString() 
		{
			if (title != null) 
			{
				return title;
			} 
			else 
			{
				return base.ToString();
			}
		}


		// Factory //////////////////////////////////

		internal static void load(string name) 
		{
			Music[] buffer = new Music[1000];
			int count;
			try 
			{
				StreamReader input = new StreamReader(name);
				input.ReadLine(); // skip column headings
				for (count=0; count<buffer.Length && input.Peek() != -1; count++) 
				{
					buffer[count]=parse(input.ReadLine());
				}
				input.Close();
			} 
			catch (Exception e) 
			{
				Console.WriteLine(e);
				return;
			}
			library = new Music[count];
			for (int i=0; i<count; i++) 
			{
				library[i] = buffer[i];
			}
			return;
		}

		internal static Music parse(string text) 
		{
			Music m = new Music();
			string[] tokens = text.Split(new char[] {'\t'});
			m.title =       tokens[0];
			m.artist =      tokens[1];
			m.album =       tokens[2];
			m.genre =       tokens[3];
			m.size =        long.Parse(tokens[4]);
			m.seconds =     int.Parse(tokens[5]);
			m.trackNumber = int.Parse(tokens[6]);
			m.trackCount =  int.Parse(tokens[7]);
			m.year =        int.Parse(tokens[8]);
			m.date =        DateTime.Parse(tokens[9]);
			return m;
		}


		// Library //////////////////////////////////

		internal static void select(Music m) 
		{
			looking = m;
		}

		internal static void play(Music m) 
		{
			// nothing yet
		}

		internal static void findAll() 
		{
			for (int i=0; i<library.Length; i++) 
			{
				library[i].selected = true;
			}
		}

		internal static void findArtist(string a) 
		{
			for (int i=0; i<library.Length; i++) 
			{
				library[i].selected = library[i].artist == a;
			}
		}

		internal static void findAlbum(string a) 
		{
			for (int i=0; i<library.Length; i++) 
			{
				library[i].selected = library[i].album == a;
			}
		}

		internal static void findGenre(string a) 
		{
			for (int i=0; i<library.Length; i++) 
			{
				library[i].selected = library[i].genre == a;
			}
		}

		internal static void findYear(int a) 
		{
			for (int i=0; i<library.Length; i++) 
			{
				library[i].selected = library[i].year == a;
			}
		}

		internal static int displayCount() 
		{
			int count = 0;
			for (int i=0; i<library.Length; i++) 
			{
				count += (library[i].selected ? 1 : 0);
			}
			return count;
		}

		internal static Music[] displayContents () 
		{
			Music[] displayed = new Music[displayCount()];
			for (int i=0, j=0; i<library.Length; i++) 
			{
				if (library[i].selected) 
				{
					displayed[j++] = library[i];
				}
			}
			return displayed;
		}
	}
}