// Copyright (c) 2002 Cunningham & Cunningham, Inc.
// Released under the terms of the GNU General Public License version 2 or later.

using System;
using System.IO;

namespace fit
{
	public class FileRunner 
	{

		protected internal string input;
		protected internal Parse tables;
		protected internal Fixture fixture = new Fixture();
		protected internal TextWriter output;

		public static void Main (String[] argv) 
		{
			new FileRunner().run(argv);
		}

		public virtual void run (String[] argv) 
		{
			args(argv);
			try 
			{
				tables = new Parse(input);
				fixture.doTables(tables);
			} 
			catch (Exception e) 
			{
				exception(e);
			}
			tables.print(output);
			exit();
		}

		protected internal void args(String[] argv) 
		{
			if (argv.Length != 2 && argv.Length != 3) 
			{
				Console.Error.WriteLine("usage: runFile input-file output-file [fixture-dir]");
				Console.Error.WriteLine("fixture-dir: The directory containing your fixture DLL.  You can specify");
				Console.Error.WriteLine("  multiple directories by separating them with semicolons.  If you leave this");
				Console.Error.WriteLine("  out, runFile will look in a number of default locations.");
				Console.Error.WriteLine("Example: runFile doc\\arithmetic.html report\\arithmetic.html bin");
				Console.Error.WriteLine("Default fixture-dir:");
				Console.Error.WriteLine("  " + fixturePath);
				Environment.Exit(-1);
			}
			string inFile = argv[0];
			string outFile = argv[1];
			if (argv.Length > 2)
			{
				string fixtureDir = argv[2];
				if (fixtureDir.ToLower().IndexOf(".dll") >= 0)
				{
					Console.Error.WriteLine("Warning: fixture-dir parameter should only contain directories");
				}
				Fixture.assemblyDirs = argv[2].Split(';');
			}
			Fixture.summary["input file"] = inFile;
			Fixture.summary["input update"] = File.GetLastWriteTime(inFile);
			Fixture.summary["output file"] = outFile;
			Fixture.summary["fixture path"] = fixturePath;
			try 
			{
				input = read(inFile);
				output = new StreamWriter(outFile);
			} 
			catch (IOException e) 
			{
				Console.Error.WriteLine(e.Message);
				Environment.Exit(-1);
			}
		}

		private string fixturePath
		{
			get
			{
				return string.Join(";", Fixture.assemblyDirs);
			}
		}

		protected internal string read(string input) 
		{
			StreamReader reader = new StreamReader(input);
			try
			{
				return reader.ReadToEnd();
			}
			finally
			{
				reader.Close();
			}
		}

		protected internal void exception(Exception e) 
		{
			tables = new Parse("body","Unable to parse input. Input ignored.", null, null);
			Fixture.exception(tables, e);
		}

		protected internal void exit() 
		{
			output.Close();
			Console.Error.WriteLine(Fixture.counts());
			Environment.Exit(Fixture.numberWrong + Fixture.numberExceptions);
		}

	}
}
