// Copyright (c) 2002 Cunningham & Cunningham, Inc.
// Released under the terms of the GNU General Public License version 2 or later.

using System;
using System.Reflection;

namespace fit
{
	public class TypeAdapter 
	{
		public object target;
		public Fixture fixture;
		public FieldInfo field;
		public MethodInfo method;
		public Type type;

		private TypeAdapter()
		{
		}

		//
		//
		//		// Factory //////////////////////////////////
		//
		public static TypeAdapter on(Fixture target, Type type)
		{
			TypeAdapter adapter = new TypeAdapter();
			adapter.fixture = target;
			adapter.target = target;
			adapter.type = type;
			return adapter;
		}

		public static TypeAdapter on(Fixture target, MethodInfo method)
		{
			TypeAdapter adapter = on(target, method.ReturnType);
			adapter.method = method;
			return adapter;
		}

		public static TypeAdapter on(Fixture target, FieldInfo field)
		{
			TypeAdapter adapter = on(target, field.FieldType);
			adapter.field = field;
			return adapter;
		}

		public object get()
		{
			if (field != null) return field.GetValue(target);
			if (method != null) return method.Invoke(target, new object[] {});
			return null;
		}

		public void set(object value)
		{
			field.SetValue(target, value);
		}
		

		public object parse(string s)
		{
			BindingFlags flags = BindingFlags.Static | BindingFlags.FlattenHierarchy
				| BindingFlags.IgnoreCase | BindingFlags.Public;
			MethodInfo parseMethod = type.GetMethod("Parse", flags, null, new Type[] {typeof(string)}, null);
			if (parseMethod != null && parseMethod.ReturnType == type)
			{
				return parseMethod.Invoke(null, new object[] {s});
			}
			else if (type.IsAssignableFrom(typeof(string)))
			{
				return s;
			}
			else
			{
				throw new ApplicationException("Don't know how to parse " + type.FullName);
			}
		}

		
		
		//		// Subclasses ///////////////////////////////
		//
		//		class ArrayAdapter 
		//		{
		//			Type componentType;
		//			TypeAdapter componentAdapter;
		//
		//			void init(Fixture target, Type type) 
		//			{
		//				super.init(target, type);
		//				componentType = type.getComponentType();
		//				componentAdapter = on(target, componentType);
		//			}
		//
		//		public object parse(string s) 
		//		{
		//			StringTokenizer t = new StringTokenizer(s, ",");
		//			object array = Array.newInstance(componentType, t.countTokens());
		//			for (int i=0; t.hasMoreTokens(); i++) 
		//			{
		//				Array.set(array, i, componentAdapter.parse(t.nextToken()));
		//			}
		//			return array;
		//		}
		//
		//		public string toString(object o) 
		//		{
		//			int length = Array.getLength(o);
		//			StringBuffer b = new StringBuffer(5*length);
		//			for (int i=0; i<length; i++) 
		//			{
		//				b.append(componentAdapter.toString(Array.get(o, i)));
		//				if (i < (length-1)) 
		//				{
		//					b.append(',');
		//				}
		//			}
		//			return b.toString();
		//		}
		//
		//		public bool equals(object a, object b) 
		//		{
		//			int length = Array.getLength(a);
		//			if (length != Array.getLength(b)) return false;
		//			for (int i=0; i<length; i++) 
		//			{
		//				if (!componentAdapter.equals(Array.get(a,i), Array.get(b,i))) return false;
		//			}
		//			return true;
		//		}
		//	}
	}
}